interface TypeProps {
}
type TypeNames = string;
interface Type<T = unknown> {
    type: TypeNames;
    _value?: T;
    _optional?: boolean;
    _default?: T | (() => T);
    _object?: Record<string, Type>;
    _union?: Type[];
    _check: (obj: any) => boolean;
    optional: () => Type<T | undefined>;
    default: (value: any) => Type<T | undefined>;
    parse: (obj: any, opt?: {
        transform?: boolean;
        strict?: boolean;
    }) => T;
    _props?: TypeProps;
    props: (props: TypeProps) => Type<T>;
    map: (obj: any, fn: (this: Type<T>, obj: any, schema: Type<T>) => any) => any;
}
type Infer<T> = T extends Type<infer TT> ? TT : never;
type ObjectFixOptional<T> = {
    [K in keyof T as undefined extends T[K] ? K : never]?: T[K] & {};
} & {
    [K in keyof T as undefined extends T[K] ? never : K]: T[K] & {};
};
type ObjectPretty<V> = Extract<{
    [K in keyof V]: V[K];
}, unknown>;
type TypeObject<T = unknown> = Type<ObjectPretty<ObjectFixOptional<{
    [K in keyof T]: Infer<T[K]>;
}>>>;
declare function string(): Type<string>;
declare function number(): Type<number>;
declare const float: typeof number;
declare function int(): Type<number>;
declare function boolean(): Type<boolean>;
declare function object<T>(tobj: T): TypeObject<T>;
type TransformToUnion<T extends (Type<any>)[]> = T extends Array<infer U> ? Infer<U> : never;
declare function union<T extends (Type<any>)[]>(options: T): Type<TransformToUnion<T>>;
type Literal = string | number | bigint | boolean;
declare function literal<T extends Literal>(value: T): Type<T>;
declare function stringLiterals<const T extends readonly string[], O = T[number]>(value: T): Type<O>;
type TupleOutput<T extends Type[]> = {
    [K in keyof T]: T[K] extends Type<infer U> ? U : never;
};
type ArrayOutput<Head extends Type[], Rest extends Type | undefined> = [
    ...TupleOutput<Head>,
    ...(Rest extends Type ? Infer<Rest>[] : [])
];
type ArrayType<Head extends Type[] = Type[], Rest extends Type | undefined = Type | undefined> = Type<ArrayOutput<Head, Rest>>;
declare function tuple<T extends [] | [Type, ...Type[]]>(items: T): ArrayType<T, undefined>;
declare function func<Args extends [Type<unknown>, ...Type<any>[]] | [], Ret = Type, T = (...args: TupleOutput<Args>) => Infer<Ret>>(args: Args, ret: Ret): Type<T>;

export { type Infer, type Type, type TypeNames, type TypeObject, type TypeProps, boolean, float, func, int, literal, number, object, string, stringLiterals, tuple, union };
